'use client';

import React, { useCallback, useState } from 'react';
import { Button, Modal, notification } from 'antd';
import { CloseOutlined } from '@ant-design/icons';
import debounce from 'lodash/debounce';
import Image from 'next/image';
import { i18nText } from '../../i18nKeys';
import { getUsersList } from '../../actions/rooms';
import { PublicUser } from '../../types/sessions';
import { Room } from '../../types/rooms';
import { CustomInput } from '../view/CustomInput';
import { Loader } from '../view/Loader';

type UserListModalProps = {
    room: Room;
    isOpen: boolean;
    locale: string;
    handleCancel: () => void;
    jwt: string;
    submit: (id: number) => void;
    afterCloseModal?: () => void;
};

export const UserListModal = ({ room, isOpen, locale, handleCancel, jwt, submit, afterCloseModal }: UserListModalProps) => {
    const [users, setUsers] = useState<PublicUser[] | undefined>();
    const [loading, seLoading] = useState<boolean>(false);

    const onSearch = useCallback(debounce((e: any) => {
        if (e?.target?.value) {
            seLoading(true);
            getUsersList(locale, jwt, { template: e.target.value })
                .then(({ items }) => {
                    const clients = room?.clients?.map(({ id }) => id);
                    setUsers(items
                        ? items.filter(({ id }) => !(clients?.length && clients.includes(id) || id === room?.supervisor?.id || id === room?.coach?.id))
                        : undefined);
                })
                .catch((err: any) => {
                    notification.error({
                        message: 'Error',
                        description: err?.response?.data?.errMessage
                    });
                })
                .finally(() => {
                    seLoading(false);
                });
        } else {
            setUsers(undefined);
        }

    }, 300), []);

    const onAfterClose = () => {
        setUsers(undefined);
        if (afterCloseModal) afterCloseModal();
    }

    return (
        <Modal
            className="b-modal"
            open={isOpen}
            title={undefined}
            onOk={undefined}
            onCancel={handleCancel}
            footer={false}
            width={498}
            closeIcon={<CloseOutlined style={{ fontSize: 20, color: '#000' }}/>}
            afterClose={onAfterClose}
        >
            <div className="b-modal__users-list__content">
                <CustomInput
                    placeholder={i18nText('search', locale)}
                    onChange={onSearch}
                    allowClear
                />
                {users && (
                    <div className="b-users-list__wrapper">
                        <Loader isLoading={loading}>
                            {users.length > 0 ? (
                                <div className="b-users-list">
                                    {users.map(({ id, name, surname, faceImageUrl }) => (
                                        <div className="b-users-list-item" key={id}>
                                            <div>
                                                <div className="card-detail__portrait card-detail__portrait_small">
                                                    <Image src={faceImageUrl || '/images/user-avatar.png'} width={86}
                                                           height={86}
                                                           alt=""/>
                                                </div>
                                                <div className="card-detail__inner">
                                                    <div
                                                        className="card-detail__name">{`${name} ${surname || ''}`}</div>
                                                </div>
                                            </div>
                                            <Button
                                                className="card-detail__filled"
                                                onClick={() => submit(id)}
                                            >
                                                {i18nText('room.invite', locale)}
                                            </Button>
                                        </div>
                                    ))}
                                </div>
                            ) : (
                                <div className="b-users-list__empty">{i18nText('noData', locale)}</div>
                            )}
                        </Loader>
                    </div>
                )}
            </div>
        </Modal>
    )
}